-- GridSmartNames by Crankshaft
-- Much code "appropriated" from GridStatusName and GridFriendlyNames

local GridRoster = Grid:GetModule("GridRoster")
local GridFrame = Grid:GetModule("GridFrame")
local GridStatus = Grid:GetModule("GridStatus")
local GridSmartNames = GridStatus:NewModule("GridSmartNames")

GridSmartNames.menuName = "Smart Names"

-- Default Option Values Go here
GridSmartNames.defaultDB = {
   debug = false,
   smart_name = {
      text = "Smart Name",
      enable = true,
      color = { r = 1, g = 1, b = 1, a = 1 },
      priority = 1,
      class = true,
      border = true,
      range = false
   },
   aliases = {}
}


GridSmartNames.currentColor = { }


GridSmartNames.options = false

function GridSmartNames:Menu()
   -- Inside the return { } put ace3 config options
   -- see also http://www.wowace.com/addons/ace3/pages/ace-config-3-0-options-tables/
   local opts = {
      ["class"] = {
	 name = "Use Class Color",
	 desc = "Color by class",
	 type = 'toggle',
	 get = function() return GridSmartNames.db.profile.smart_name.class end,
	 set = function()
		  GridSmartNames.db.profile.smart_name.class = not GridSmartNames.db.profile.smart_name.class
		  GridSmartNames:UpdateAllUnits()
	       end
      },
      ["border"] = {
	 name = "Use Border Status Color",
	 desc = "Override the color with the color of the current border status.",
	 type = 'toggle',
	 get = function() return GridSmartNames.db.profile.smart_name.border end,
	 set = function()
		  GridSmartNames.db.profile.smart_name.border = not GridSmartNames.db.profile.smart_name.border
		  GridSmartNames:UpdateAllUnits()
	       end
      },
      ["aliases"] = {
	 type = "group",
	 name = "Aliases",
	 desc = "Settings for individual Aliases",
	 args = {
	    ["add_alias"] = {
	       type = "input",
	       name = "Add a new Alias",
	       desc = "Enter the name of a character to add an alias for. Spelling and capitalization count.",
	       get = false,
	       set = function(_, v) self:AddAlias(v) end
	    },
	 }
      },
      ["delete_alias"] = {
	 type = "group",
	 name = "Delete an Alias",
	 desc = "Delete an existing alias competely.",
	 args = {},
	 order = -1
      }
   }
   
   for alias, aliasTbl in pairs(self.db.profile.aliases) do
      local alias = alias
      if type(aliasTbl) == "table" and aliasTbl.alias and not self.defaultDB.aliases[alias] then
	 opts["aliases"].args[alias] = {
	    type = "group",
	    name = alias,
	    desc = string.format("Edit %s's alias", alias),
	    args = GridSmartNames:addAliasOptions(alias)
	 }
	 opts["delete_alias"].args[alias] = {
	    type = "execute",
	    name = alias,
	    desc = string.format("Remove %s from the alias list", alias),
	    func = function() return self:DeleteAlias(alias) end
	 }
      end
   end
   return opts
end

function GridSmartNames:OnInitialize()
   self.super.OnInitialize(self)
   local menu = self:Menu()
   -- args are status, text, options menu, inMainMenu
   self:RegisterStatus("smart_name", "Smart Names", menu, true)

   -- we're going to listen to the border indicator
   hooksecurefunc(GridFrame.prototype, "ClearIndicator", self.ClearIndicator)
   hooksecurefunc(GridFrame.prototype, "SetIndicator", self.SetIndicator)
end

function GridSmartNames:OnStatusEnable(status)
   if status == "smart_name" then
      self:Debug('Enabled')
      self:RegisterEvent("UNIT_NAME_UPDATE", "UpdateUnit")
      self:RegisterEvent("UNIT_PORTRAIT_UPDATE", "UpdateUnit")
      self:RegisterEvent("UNIT_ENTERED_VEHICLE", "UpdateVehicle")
      self:RegisterEvent("UNIT_EXITED_VEHICLE", "UpdateVehicle")
      self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateAllUnits")
      
      self:RegisterMessage("Grid_UnitJoined", "UpdateGUID")
      self:RegisterMessage("Grid_UnitChanged", "UpdateGUID")
      self:RegisterMessage("Grid_UnitLeft", "UpdateGUID")
      
      self:RegisterMessage("Grid_ColorsChanged", "UpdateAllUnits")
      
      self:UpdateAllUnits()
   end
end

function GridSmartNames:OnStatusDisable(status)
	if status == "smart_name" then
	   self:Debug('Disabled')
		self:UnregisterEvent("UNIT_NAME_UPDATE")
		self:UnregisterEvent("UNIT_PORTRAIT_UPDATE")
		self:UnregisterEvent("UNIT_ENTERED_VEHICLE")
		self:UnregisterEvent("UNIT_EXITED_VEHICLE")
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")

		self:UnregisterMessage("Grid_UnitJoined")
		self:UnregisterMessage("Grid_UnitChanged")
		self:UnregisterMessage("Grid_UnitLeft")
		self:UnregisterMessage("Grid_ColorsChanged")

		self.core:SendStatusLostAllUnits("smart_name")
	end
end

function GridSmartNames.SetIndicator(frame, indicator, color, text, value, maxValue, texture, start, duration, stack)
   if indicator == "border" then
      GridSmartNames.currentColor[frame.unitGUID] = color
      GridSmartNames:Debug("color set", color.r, color.g, color.b, color.a)
      GridSmartNames:UpdateGUID({}, frame.unitGUID)
   end
end

function GridSmartNames.ClearIndicator(frame, indicator)
   if indicator == "border" then
      GridSmartNames:Debug("border clear")
      if (frame.unitGUID) then
	 GridSmartNames:Debug("Got GUID", frame.unitGUID)
	 local guid = frame.unitGUID
	 local settings = GridSmartNames.db.profile.smart_name or GridSmartNames.defaultDB
	 local color = settings.class and GridSmartNames.core:UnitColor(guid) or settings.color
	 GridSmartNames:Debug("color set", color.r, color.g, color.b, color.a)
	 GridSmartNames.currentColor[guid] = color
	 GridSmartNames:UpdateGUID({}, guid)
      end
   end
end

function GridSmartNames:Reset()
   self.super.Reset(self)
   self:UpdateAllUnits()
end

function GridSmartNames:UpdateVehicle(event, unitid)
   self:UpdateUnit(event, unitid)
   local pet_unit = unitid .. "pet"
   if UnitExist(pet_unit) then
      self:UpdateUnit(event, pet_unit)
   end
end

function GridSmartNames:UpdateUnit(event, unitid)
	self:UpdateGUID(event, UnitGUID(unitid))
end

function GridSmartNames:UpdateGUID(event, guid)
	local settings = self.db.profile.smart_name

	local name = GridRoster:GetNameByGUID(guid)


	if not name or not settings.enable then return end


	-- set text
	local text = name

	
	if self.db.profile.aliases[name] and self.db.profile.aliases[name].enable then
	text = self.db.profile.aliases[name].alias
	end

	local show_owner_name = true
	if show_owner_name then
		local unitid = GridRoster:GetUnitidByGUID(guid)
		local owner_unitid = GridRoster:GetOwnerUnitidByUnitid(unitid)

		-- does this unit have an owner?
		-- is the owner driving a vehicle?
		if owner_unitid and UnitHasVehicleUI(owner_unitid) then
			local owner_guid = UnitGUID(owner_unitid)
			local owner_name = GridRoster:GetNameByGUID(owner_guid)

			text = owner_name
		end
	end

	-- set color
	local color = settings.class and self.core:UnitColor(guid) or settings.color
	if settings.border then
	   if self.currentColor[guid] then
	      color = self.currentColor[guid]
	   end
	end

	self.core:SendStatusGained(guid, "smart_name",
		settings.priority,
		nil,
		color,
		text)
end

function GridSmartNames:UpdateAllUnits()
	for guid, unitid in GridRoster:IterateRoster() do
		self:UpdateGUID("UpdateAllUnits", guid)
	end
end

function GridSmartNames:addAliasOptions(name)
   local opts = { 
      ["alias"] = {
	 type = "input",
	 name = "Add alias",
	 desc = "Sets the alias for this character which will be displayed.",
	 get = function() return GridSmartNames.db.profile.aliases[name].alias end,
	 set = function(_, v) 
		  GridSmartNames.db.profile.aliases[name].alias = v
		  GridSmartNames:UpdateAllUnits()
	       end
      },
      ["enable"] = {
	 type = "toggle",
	 name = "Enable",
	 desc = "Enable this alias",
	 get = function() return GridSmartNames.db.profile.aliases[name].enable end,
	 set = function()
		  GridSmartNames.db.profile.aliases[name].enable = not GridSmartNames.db.profile.aliases[name].enable 
		  GridSmartNames:UpdateAllUnits()
	       end
      }
   }
   return opts
end


function GridSmartNames:AddAlias(alias)
   GridSmartNames:Debug("adding alias",alias)
   self.db.profile.aliases[alias] = {
      alias = alias,
      enable = true
   }
   
   GridSmartNames:Debug('add', alias, self.options)
   GridStatus.options.args['smart_name'].args.aliases.args[alias] = {
	    type = "group",
	    name = alias,
	    desc = string.format("Edit %s's alias", alias),
	    args = GridSmartNames:addAliasOptions(alias)
	 }
   GridStatus.options.args['smart_name'].args["delete_alias"].args[alias] = {
	    type = "execute",
	    name = alias,
	    desc = string.format("Remove %s from the alias list", alias),
	    func = function() return self:DeleteAlias(alias) end
	 }
   GridSmartNames:UpdateAllUnits()
end

function GridSmartNames:DeleteAlias(alias)
   self.db.profile.aliases[alias] = nil
   GridStatus.options.args['smart_name'].args.aliases.args[alias] = nil
   GridStatus.options.args['smart_name'].args["delete_alias"].args[alias] = nil
   GridSmartNames:Debug('delete', alias, self.options)
   GridSmartNames:UpdateAllUnits()
end
